from .result import Result, BatchResult, SingleResult, FileOperator
from .task_result import TaskResult

__all__ = [
    "FileOperator",
    "TaskResult",
    "Result",
    "SingleResult",
    "BatchResult",
]
